<?php

function read_gpei_xml($config, $gpei, &$users, $uai = "")
{
    // read entire file into string
    $xmlfile = file_get_contents($gpei);

    $xml = new DOMDocument();
    $xml->formatOutput = false;
    $xml->preserveWhiteSpace = false;
    $xml->loadXML($xmlfile);
    $xpath = new DOMXPath($xml);

    $users = array_merge($users, read_attrs($xpath->query('/ficAlimMENESR/addRequest/attributes/attr')));
    $users = array_merge($users, read_attrs($xpath->query('/ficAlimMENESR/modifyRequest/modifications/modification')));

    foreach ($xpath->query('/ficAlimMENESR/deleteRequest/identifier/id') as $id) {
        $users[] = [
            'Id GPEI' => $id->nodeValue,
            'type' => "trash"
        ];
    }
    if (! empty($uai) && $uai != 0) {
        foreach ($users as $id => $user) {
            $found = false;
            if (isset($user['etabs']) && is_array($user['etabs'])) {
                foreach ($user['etabs'] as $i) {
                    $found |= ($uai == etab_uai($config, $i));
                }
            }
            if (! $found) {
                unset($users[$id]);
            }
        }
    }

    return count($users);
}

function read_attrs($reqs)
{
    $user = [];
    $i = 0;
    foreach ($reqs as $i => $req) {
        // if ($i ++ > 100)
        // break;
        $id = $req->parentNode->parentNode->getElementsByTagName('identifier')
            ->item(0)
            ->getElementsByTagName('id')
            ->item(0)->nodeValue;
        $cat = $req->parentNode->parentNode->getElementsByTagName('operationalAttributes')
            ->item(0)
            ->getElementsByTagName('attr')
            ->item(0)->nodeValue;
        switch ($cat) {
            case "PersEducNat":
                $user[$id]['type'] = "Teacher";
                break;
            case "Eleve":
                $user[$id]['type'] = "Student";
                break;
            case "EtabEducNat":
                $user[$id]['type'] = "Etab";
                break;
        }
        $attr = $req->getAttribute('name');
        $vals = $req->getElementsByTagName('value');
        foreach ($vals as $v) {
            if (is_object($v->firstChild)) {
                $val = $v->firstChild->nodeValue;
                switch ($attr) {
                    case "GPEIPersonDateNaissance":
                        $key = "birthDate";
                        $user[$id][$key] = map_birthdate($val, "gpei");
                        break;
                    case "GPEIPersonJointure":
                    case "GPEIStructureJointure":
                        $key = "Id GPEI";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonNom":
                        $key = "lastName";
                        $user[$id][$key] = $val;
                        $user[$id]['lastName_norm'] = simplify_name($val);
                        break;
                    case "GPEIPersonPrenom":
                        $key = "firstName";
                        $user[$id][$key] = $val;
                        $user[$id]['firstName_norm'] = simplify_firstname($val);
                        break;
                    case "GPEIPersonMail":
                    case "GPEIStructureMailSI":
                        $key = "email";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonStructRattach":
                        $key = "structure";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIEleveStructRattachId":
                        $key = "Id Siecle";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonFonctions":
                        $tab = explode('$', $val);
                        $key = "functions";
                        $user[$id][$key][] = [
                            'etab' => $tab[0],
                            'fonc' => $tab[1], // DIR, ADF, EDU, AED, MDS
                            'fonction' => $tab[2],
                            'id' => $tab[3],
                            'discipline' => $tab[4]
                        ];
                        break;
                    case "GPEIEnsDiv":
                    case "GPEIElvDiv":
                        $key = "allClasses";
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'name' => $tab[1],
                                'description' => $tab[2] ?? $tab[1]
                            ];
                        }
                        break;
                    case "GPEIEnsGpes":
                    case "GPEIElvGpes":
                        $key = "allgroups";
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'name' => $tab[1],
                                'description' => $tab[2] ?? $tab[1]
                            ];
                        }
                        break;
                    case "GPEIEnsMatieres":
                    case "GPEIElvMatieres":
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $key = "allMatieres";
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'id' => $tab[1],
                                'name' => $tab[2]
                            ];
                        }
                        break;
                    case "GPEIStructureUAI":
                        $key = "UAI";
                        $user[$id][$key] = strtolower($val);
                        break;
                    case "GPEIStructureNomCourant":
                        $key = "name";
                        $user[$id][$key] = explode("$", $val)[1];
                        break;
                    case "GPEIStructureTypeStruct":
                        $key = "typeEtab";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIServAcAcademie":
                        $key = "academie";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureVille":
                        $key = "city";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureCodePostal":
                        $key = "code";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureTelephone":
                        $key = "phone";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIEtablissementBassin":
                        $tab = explode('$', $val);
                        if (count($tab) > 0) {
                            $key = "bassin";
                            $user[$id][$key] = [
                                'id' => $tab[0],
                                'name' => $tab[1]
                            ];
                        }
                        break;
                }
            }
        }
        if (isset($user[$id]['structure'])) {
            $user[$id]['etabs'][0] = $user[$id]['structure'];
            if (isset($user[$id]['functions'])) {
                foreach ($user[$id]['functions'] as $fonction) {
                    if ($fonction['etab'] == $user[$id]['structure']) {
                        // etablissement de rattachement
                        switch ($fonction['fonc']) {
                            case 'ENS':
                                $fonc = true;
                                $user[$id]['fonction'] = "";
                                $user[$id]['type'] = "Teacher";
                                break;
                            case 'DOC': // Documentaliste
                                $fonc = true;
                                $user[$id]['fonction'] = "Documentaliste";
                                $user[$id]['type'] = "Teacher";
                                break;
                            case 'DIR': // principal, adjoint, gestionnaire
                                $fonc = true;
                                $user[$id]['fonction'] = "Direction";
                                $user[$id]['type'] = "Personnel";
                                break;
                            case 'ADF': // secrétaires
                                $fonc = true;
                                $user[$id]['fonction'] = "Secretariat";
                                $user[$id]['type'] = "Personnel";
                                break;
                            case 'MDS': // medecin, infirmier
                            case 'AES': // AESH
                                $fonc = true;
                                $user[$id]['fonction'] = "Medical";
                                $user[$id]['type'] = "Personnel";
                                break;
                            case 'AED': // AED
                            case 'EDU': // CPE
                                $fonc = true;
                                $user[$id]['fonction'] = "VieScol";
                                $user[$id]['type'] = "Personnel";
                                break;
                            default:
                                $fonc = false;
                        }
                    } elseif (! in_array($fonction['etab'], $user[$id]['etabs'])) {
                        // autre établissement
                        $user[$id]['etabs'][] = $fonction['etab'];
                    }
                    if (! $fonc && $fonction['fonc'] == "-") {
                        $user[$id]['fonction'] = "Agent";
                        $user[$id]['type'] = "Personnel";
                    }
                }
            }
        }
    }
    return $user;
}

function sync_gpei($config, &$html = "")
{
    $html .= "<h1>Import gpei</h1>\n";
    $dossier_tmp_import_comptes = sys_get_temp_dir() . "/gpei";
    if (! is_dir($dossier_tmp_import_comptes)) {
        mkdir($dossier_tmp_import_comptes, "0770");
    }
    if (! empty($config['gpei_url'])) {
        // on est sur le central
        $ssh = "/usr/bin/ssh -i /etc/sambaedu/id_rsa " . $config['gpei_url'];
        $list = [];
        $html .= exec($ssh . " ls *.zip", $list);
        // recherche du dernier complet et des derniers deltas
        $complet = $delta = [];
        foreach ($list as $k => $gpei) {
            $m = [];
            if (preg_match("/^.*_GPEI_(Complet|Delta)_([0-9]{8})\.zip$/", $gpei, $m)) {
                if ($m[1] == "Complet") {
                    $complet[$k] = $gpei;
                } else {
                    $delta[$k] = $gpei;
                }
            }
        }
        sort($complet);
        if (count($complet) > 0) {
            // on prend le plus récent complet pour commencer
            $file = array_pop($complet);
            // on efface les anciens complets qui ne servent à rien
            foreach ($complet as $k => $c) {
                if (file_exists($dossier_tmp_import_comptes . "/" . $c)) {
                    unlink($dossier_tmp_import_comptes . "/" . $c);
                }
                unset($list[$k]);
            }
            $m = [];
            if (preg_match("/^.*_GPEI_Complet_([0-9]{8})\.zip$/", $file, $m)) {
                $date = $m[1];
                foreach ($delta as $k => $d) {
                    $m = [];
                    if (preg_match("/^.*_GPEI_Delta_([0-9]{8})\.zip$/", $d, $m)) {
                        // on efface les anciens deltas
                        if ($m[1] < $date) {
                            if (file_exists($dossier_tmp_import_comptes . "/" . $d)) {
                                unlink($dossier_tmp_import_comptes . "/" . $d);
                            }
                            unset($list[$k]);
                        }
                    }
                }
            }
        }
        // liste des derniers fichers téléchargés
        if (file_exists($dossier_tmp_import_comptes . "/list.json")) {
            $oldlist = json_decode(file_get_contents($dossier_tmp_import_comptes . "/list.json"), true);
            foreach ($oldlist as $k => $f) {
                if (! preg_match("/^.*_GPEI_.*_[0-9]{8}\.zip$/", $f)) {
                    unset($oldlist[$k]);
                }
            }
        } else {
            $oldlist = [];
        }
        // $html .= print_r($oldlist, true);
        // $html .= print_r($list, true);
        $new = array_diff($list, $oldlist);
        // $html .= "Récupération de " . count($new) . " fichiers gpei<br>\n";
        // $html .= print_r($new, true);
        // return $html;
        foreach ($new as $gpei) {
            $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null " . $config['gpei_url'] . ":~/" . $gpei . " " . $dossier_tmp_import_comptes;
            system($scp);
        }

        $dl_list = scandir($dossier_tmp_import_comptes);
        // on retire les intrus
        foreach ($dl_list as $k => $f) {
            if (! preg_match("/^.*_GPEI_.*_[0-9]{8}\.zip$/", $f)) {
                unset($dl_list[$k]);
            }
        }
        foreach (activated_etabs($config) as $etab) {
            if ($etab['activated'] && fping("se4fs-" . $etab['uai'])) {
                system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " mkdir -p " . $dossier_tmp_import_comptes . "/" . $etab['uai']);
                system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " chown -R www-admin " . $dossier_tmp_import_comptes . "/" . $etab['uai']);
                system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " chmod 600 " . $dossier_tmp_import_comptes . "/*");
                if ($n = count($dl_list) > 0) {
                    $html .= " envoi au serveur " . $etab['uai'] . " de $n fichiers<br>\n";
                    foreach ($dl_list as $gpei) {
                        $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null " . $dossier_tmp_import_comptes . "/" . $gpei . " root@se4fs-" . $etab['uai'] . ":" . $dossier_tmp_import_comptes . "/" . $etab['uai'] . "/" . $gpei;
                        system($scp);
                    }
                }
            } else {
                if (! is_dir($dossier_tmp_import_comptes . "/" . $etab['uai'])) {
                    mkdir($dossier_tmp_import_comptes . "/" . $etab['uai']);
                }
                foreach ($dl_list as $gpei) {
                    copy($dossier_tmp_import_comptes . "/" . $gpei, $dossier_tmp_import_comptes . "/" . $etab['uai'] . "/" . $gpei);
                }
            }
        }
        // on supprime les fichiers utilisés
        foreach ($dl_list as $gpei) {
            unlink($dossier_tmp_import_comptes . "/" . $gpei);
        }
        // on ecrit la liste pour la prochaine fois
        file_put_contents($dossier_tmp_import_comptes . "/list.json", json_encode($list));
    }

    // serveurs de site, ou etabs non joignables sur le central
    foreach (scandir($dossier_tmp_import_comptes) as $uai) {
        $file = "";
        if (preg_match("/[0-9]{7}[a-z]/", $uai)) {
            $html .= "Recherche des fichiers GPEI restant à traiter pour $uai :";
            $dossier_tmp_import_comptes_uai = $dossier_tmp_import_comptes . "/" . $uai;
            $list = scandir($dossier_tmp_import_comptes_uai);
            $complet = $delta = [];
            $cd = 0;
            foreach ($list as $gpei) {
                $m = [];
                if (preg_match("/^.*_GPEI_(Complet|Delta)_([0-9]{8})\.zip$/", $gpei, $m)) {
                    if ($m[1] == "Complet") {
                        $complet[] = $gpei;
                    } else {
                        $delta[] = $gpei;
                        $cd ++;
                    }
                }
            }
            sort($complet);
            if (count($complet) > 0) {
                // on prend le plus récent complet pour commencer
                $file = array_pop($complet);
                // on efface les anciens complets qui ne servent à rien
                foreach ($complet as $c) {
                    unlink($dossier_tmp_import_comptes_uai . "/" . $c);
                }
                $m = [];
                if (preg_match("/^.*_GPEI_Complet_([0-9]{8})\.zip$/", $file, $m)) {
                    $date = $m[1];
                    foreach ($delta as $d) {
                        $m = [];
                        if (preg_match("/^.*_GPEI_Delta_([0-9]{8})\.zip$/", $d, $m)) {
                            // on efface les anciens deltas
                            if ($m[1] < $date) {
                                unlink($dossier_tmp_import_comptes_uai . "/" . $d);
                                $cd --;
                            }
                        }
                    }
                }
            } elseif ($cd > 0) {
                sort($delta);
                $file = array_shift($delta);
            } else {
                $file = "";
            }

            if (! empty($file)) {
                apcu_store('uai', $uai);
                $config['etab_ou'] = $uai;
                $html .= import_archive($config, $dossier_tmp_import_comptes_uai . "/" . $file, "gpei");
                unlink($dossier_tmp_import_comptes_uai . "/" . $file);
                if ($cd > 0) {
                    // on relance le script si besoin
                    return true;
                }
            } else {
                $html .= " Aucun fichier GPEI restant à traiter<br>";
            }
        }
    }
    return false;
}
?>